@@ -6,11 +6,16 @@ import sys
 import json
 import types
 import logging
+import datetime
 import requests
 
-from .decorators import *
-from requests_oauthlib import OAuth1, OAuth1Session
+import ssl
+from requests.exceptions import ConnectionError
+from requests.packages.urllib3.exceptions import ProtocolError
 
+from .decorators import *
+from requests_oauthlib import OAuth1, OAuth1Session, OAuth2Session
+from oauthlib.oauth2 import BackendApplicationClient
 
 if sys.version_info[:2] <= (2, 7):
     # Python 2
@@ -25,6 +30,8 @@ else:
     import configparser
     from urllib.parse import parse_qs
 
+log = logging.getLogger('twarc')
+
 
 class Twarc(object):
     """
@@ -38,7 +45,7 @@ class Twarc(object):
                  access_token=None, access_token_secret=None,
                  connection_errors=0, http_errors=0, config=None,
                  profile="", protected=False, tweet_mode="extended",
-                 validate_keys=True):
+                 app_auth=False, validate_keys=True):
         """
         Instantiate a Twarc instance. If keys aren't set we'll try to
         discover them in the environment or a supplied profile. If no
@@ -57,6 +64,7 @@ class Twarc(object):
         self.last_response = None
         self.tweet_mode = tweet_mode
         self.protected = protected
+        self.app_auth = app_auth
 
         if config:
             self.config = config
@@ -72,43 +80,48 @@ class Twarc(object):
     def search(self, q, max_id=None, since_id=None, lang=None,
                result_type='recent', geocode=None, max_pages=None):
         """
-        Pass in a query with optional max_id, min_id, lang or geocode
-        and get back an iterator for decoded tweets. Defaults to recent (i.e.
-        not mixed, the API default, or popular) tweets.
+        Pass in a query with optional max_id, min_id, lang, geocode, or
+        max_pages, and get back an iterator for decoded tweets. Defaults to
+        recent (i.e. not mixed, the API default, or popular) tweets.
         """
         url = "https://api.twitter.com/1.1/search/tweets.json"
         params = {
             "count": 100,
             "q": q,
-            "include_ext_alt_text": 'true'
+            "include_ext_alt_text": 'true',
+            "include_entities": "true"
         }
+
         if lang is not None:
             params['lang'] = lang
+        if geocode is not None:
+            params['geocode'] = geocode
+        if since_id:
+            # Make the since_id inclusive, so we can avoid retrieving
+            # an empty page of results in some cases
+            params['since_id'] = str(int(since_id) - 1)
+
         if result_type in ['mixed', 'recent', 'popular']:
             params['result_type'] = result_type
         else:
             params['result_type'] = 'recent'
-        if geocode is not None:
-            params['geocode'] = geocode
 
         retrieved_pages = 0
         reached_end = False
 
         while True:
-            if since_id:
-                # Make the since_id inclusive, so we can avoid retrieving
-                # an empty page of results in some cases
-                params['since_id'] = str(int(since_id) - 1)
+
+            # note: max_id changes as results are retrieved
             if max_id:
                 params['max_id'] = max_id
 
             resp = self.get(url, params=params)
-            retrieved_pages += 1
 
+            retrieved_pages += 1
             statuses = resp.json()["statuses"]
 
             if len(statuses) == 0:
-                logging.info("no new tweets matching %s", params)
+                log.info("no new tweets matching %s", params)
                 break
 
             for status in statuses:
@@ -120,15 +133,80 @@ class Twarc(object):
                 yield status
 
             if reached_end:
-                logging.info("no new tweets matching %s", params)
+                log.info("no new tweets matching %s", params)
                 break
 
             if max_pages is not None and retrieved_pages == max_pages:
-                logging.info("reached max page limit for %s", params)
+                log.info("reached max page limit for %s", params)
                 break
 
             max_id = str(int(status["id_str"]) - 1)
 
+    def premium_search(self, q, product, environment, from_date=None,
+            to_date=None, max_results=None, sandbox=False, limit=0):
+        """
+        Search using the Premium Search API. You will need to pass in a query
+        a product (30day or fullarchive) and and environment to use. Optionally
+        you can pass in a from_date and to_date to limit the search using
+        datetime objects. If you would like to set max_results you can, or
+        you can accept the maximum results (500). If using the a sandbox 
+        environment you will want to set sandbox=True to lower the max_results
+        to 100. The limit option will cause your search to finish after it has 
+        return more than that number of tweets (0 means no limit).
+        """
+
+        if not self.app_auth:
+            raise RuntimeError(
+                "This endpoint is only available with application authentication. "
+                "Pass app_auth=True in Python or --app-auth on the command line."
+            )
+
+        if from_date and not isinstance(from_date, datetime.date):
+            raise RuntimeError("from_date must be a datetime.date or datetime.datetime object")
+        if to_date and not isinstance(to_date, datetime.date):
+            raise RuntimeError("to_date must be a datetime.date or datetime.datetime object")
+
+        if product not in ['30day', 'fullarchive']:
+            raise RuntimeError(
+                'Invalid Premium Search API product: {}'.format(product)
+            )
+
+        # set default max_results based on whether its sandboxed
+        if max_results is None:
+            if sandbox:
+                max_results = 100
+            else:
+                max_results = 500
+
+        url = 'https://api.twitter.com/1.1/tweets/search/{}/{}.json'.format(
+            product, 
+            environment
+        )
+
+        params = {
+            "query": q,
+            "fromDate": from_date.strftime('%Y%m%d%H%M') if from_date else None,
+            "toDate": to_date.strftime('%Y%m%d%H%M') if to_date else None,
+            "maxResults": max_results
+        }
+    
+        count = 0
+        stop = False
+        while not stop:
+            resp = self.get(url, params=params)
+            if resp.status_code == 200:
+                data = resp.json()
+                for tweet in data['results']:
+                    count += 1
+                    yield tweet
+                    if limit != 0 and count >= limit:
+                        stop = True
+                        break
+                if 'next' in data:
+                    params['next'] = data['next']
+                else:
+                    stop = True
+
     def timeline(self, user_id=None, screen_name=None, max_id=None,
                  since_id=None, max_pages=None):
         """
@@ -139,16 +217,19 @@ class Twarc(object):
 
         if user_id and screen_name:
             raise ValueError('only user_id or screen_name may be passed')
-        elif not (user_id or screen_name):
-            raise ValueError('one of user_id or screen_name must be passed')
 
         # Strip if screen_name is prefixed with '@'
         if screen_name:
             screen_name = screen_name.lstrip('@')
         id = screen_name or str(user_id)
         id_type = "screen_name" if screen_name else "user_id"
-        logging.info("starting user timeline for user %s", id)
-        url = "https://api.twitter.com/1.1/statuses/user_timeline.json"
+        log.info("starting user timeline for user %s", id)
+
+        if screen_name or user_id:
+            url = "https://api.twitter.com/1.1/statuses/user_timeline.json"
+        else:
+            url = "https://api.twitter.com/1.1/statuses/home_timeline.json"
+
         params = {"count": 200, id_type: id, "include_ext_alt_text": "true"}
 
         retrieved_pages = 0
@@ -167,14 +248,17 @@ class Twarc(object):
                 retrieved_pages += 1
             except requests.exceptions.HTTPError as e:
                 if e.response.status_code == 404:
-                    logging.info("no timeline available for %s", id)
+                    log.warn("no timeline available for %s", id)
+                    break
+                elif e.response.status_code == 401:
+                    log.warn("protected account %s", id)
                     break
                 raise e
 
             statuses = resp.json()
 
             if len(statuses) == 0:
-                logging.info("no new tweets matching %s", params)
+                log.info("no new tweets matching %s", params)
                 break
 
             for status in statuses:
@@ -189,11 +273,11 @@ class Twarc(object):
                     yield status
 
             if reached_end:
-                logging.info("no new tweets matching %s", params)
+                log.info("no new tweets matching %s", params)
                 break
 
             if max_pages is not None and retrieved_pages == max_pages:
-                logging.info("reached max page limit for %s", params)
+                log.info("reached max page limit for %s", params)
                 break
 
             max_id = str(int(status["id_str"]) - 1)
@@ -209,7 +293,7 @@ class Twarc(object):
             raise RuntimeError("id_type must be user_id or screen_name")
 
         if not isinstance(ids, types.GeneratorType):
-            iterator = iter(ids)
+            ids = iter(ids)
 
         # TODO: this is similar to hydrate, maybe they could share code?
 
@@ -217,18 +301,18 @@ class Twarc(object):
 
         def do_lookup():
             ids_str = ",".join(lookup_ids)
-            logging.info("looking up users %s", ids_str)
+            log.info("looking up users %s", ids_str)
             url = 'https://api.twitter.com/1.1/users/lookup.json'
             params = {id_type: ids_str}
             try:
                 resp = self.get(url, params=params, allow_404=True)
             except requests.exceptions.HTTPError as e:
                 if e.response.status_code == 404:
-                    logging.warn("no users matching %s", ids_str)
+                    log.warning("no users matching %s", ids_str)
                 raise e
             return resp.json()
 
-        for id in iterator:
+        for id in ids:
             lookup_ids.append(id.strip())
             if len(lookup_ids) == 100:
                 for u in do_lookup():
@@ -239,7 +323,7 @@ class Twarc(object):
             for u in do_lookup():
                 yield u
 
-    def follower_ids(self, user):
+    def follower_ids(self, user, max_pages=None):
         """
         Returns Twitter user id lists for the specified user's followers.
         A user can be a specific using their screen_name or user_id
@@ -248,24 +332,31 @@ class Twarc(object):
         user = user.lstrip('@')
         url = 'https://api.twitter.com/1.1/followers/ids.json'
 
-        if re.match('^\d+$', user):
+        if re.match(r'^\d+$', user):
             params = {'user_id': user, 'cursor': -1}
         else:
             params = {'screen_name': user, 'cursor': -1}
 
+        retrieved_pages = 0
+
         while params['cursor'] != 0:
             try:
                 resp = self.get(url, params=params, allow_404=True)
+                retrieved_pages += 1
             except requests.exceptions.HTTPError as e:
                 if e.response.status_code == 404:
-                    logging.info("no users matching %s", screen_name)
+                    log.info("no users matching %s", screen_name)
                 raise e
             user_ids = resp.json()
             for user_id in user_ids['ids']:
                 yield str_type(user_id)
             params['cursor'] = user_ids['next_cursor']
 
-    def friend_ids(self, user):
+            if max_pages is not None and retrieved_pages == max_pages:
+                log.info("reached max follower page limit for %s", params)
+                break
+
+    def friend_ids(self, user, max_pages=None):
         """
         Returns Twitter user id lists for the specified user's friend. A user
         can be specified using their screen_name or user_id.
@@ -274,17 +365,20 @@ class Twarc(object):
         user = user.lstrip('@')
         url = 'https://api.twitter.com/1.1/friends/ids.json'
 
-        if re.match('^\d+$', user):
+        if re.match(r'^\d+$', user):
             params = {'user_id': user, 'cursor': -1}
         else:
             params = {'screen_name': user, 'cursor': -1}
 
+        retrieved_pages = 0
+
         while params['cursor'] != 0:
             try:
                 resp = self.get(url, params=params, allow_404=True)
+                retrieved_pages += 1
             except requests.exceptions.HTTPError as e:
                 if e.response.status_code == 404:
-                    logging.error("no users matching %s", user)
+                    log.error("no users matching %s", user)
                 raise e
 
             user_ids = resp.json()
@@ -292,9 +386,13 @@ class Twarc(object):
                 yield str_type(user_id)
             params['cursor'] = user_ids['next_cursor']
 
+            if max_pages is not None and retrieved_pages == max_pages:
+                log.info("reached max friend page limit for %s", params)
+                break
+
     @filter_protected
-    def filter(self, track=None, follow=None, locations=None, event=None,
-               record_keepalive=False):
+    def filter(self, track=None, follow=None, locations=None, lang=[],
+               event=None, record_keepalive=False):
         """
         Returns an iterator for tweets that match a given filter track from
         the livestream of tweets happening right now.
@@ -318,51 +416,57 @@ class Twarc(object):
             params["follow"] = follow
         if locations:
             params["locations"] = locations
+        if lang:
+            # should be a list, but just in case
+            if isinstance(lang, list):
+                params['language'] = ','.join(lang)
+            else:
+                params['language'] = lang
         headers = {'accept-encoding': 'deflate, gzip'}
         errors = 0
         while True:
             try:
-                logging.info("connecting to filter stream for %s", params)
+                log.info("connecting to filter stream for %s", params)
                 resp = self.post(url, params, headers=headers, stream=True)
                 errors = 0
                 for line in resp.iter_lines(chunk_size=1024):
                     if event and event.is_set():
-                        logging.info("stopping filter")
+                        log.info("stopping filter")
                         # Explicitly close response
                         resp.close()
                         return
                     if not line:
-                        logging.info("keep-alive")
+                        log.info("keep-alive")
                         if record_keepalive:
                             yield "keep-alive"
                         continue
                     try:
                         yield json.loads(line.decode())
                     except Exception as e:
-                        logging.error("json parse error: %s - %s", e, line)
+                        log.error("json parse error: %s - %s", e, line)
             except requests.exceptions.HTTPError as e:
                 errors += 1
-                logging.error("caught http error %s on %s try", e, errors)
+                log.error("caught http error %s on %s try", e, errors)
                 if self.http_errors and errors == self.http_errors:
-                    logging.warn("too many errors")
+                    log.warning("too many errors")
                     raise e
                 if e.response.status_code == 420:
                     if interruptible_sleep(errors * 60, event):
-                        logging.info("stopping filter")
+                        log.info("stopping filter")
                         return
                 else:
                     if interruptible_sleep(errors * 5, event):
-                        logging.info("stopping filter")
+                        log.info("stopping filter")
                         return
             except Exception as e:
                 errors += 1
-                logging.error("caught exception %s on %s try", e, errors)
+                log.error("caught exception %s on %s try", e, errors)
                 if self.http_errors and errors == self.http_errors:
-                    logging.warn("too many exceptions")
+                    log.warning("too many exceptions")
                     raise e
-                logging.error(e)
+                log.error(e)
                 if interruptible_sleep(errors, event):
-                    logging.info("stopping filter")
+                    log.info("stopping filter")
                     return
 
     def sample(self, event=None, record_keepalive=False):
@@ -380,47 +484,120 @@ class Twarc(object):
         errors = 0
         while True:
             try:
-                logging.info("connecting to sample stream")
+                log.info("connecting to sample stream")
                 resp = self.post(url, params, headers=headers, stream=True)
                 errors = 0
                 for line in resp.iter_lines(chunk_size=512):
                     if event and event.is_set():
-                        logging.info("stopping sample")
+                        log.info("stopping sample")
                         # Explicitly close response
                         resp.close()
                         return
                     if line == "":
-                        logging.info("keep-alive")
+                        log.info("keep-alive")
                         if record_keepalive:
                             yield "keep-alive"
                         continue
                     try:
                         yield json.loads(line.decode())
                     except Exception as e:
-                        logging.error("json parse error: %s - %s", e, line)
+                        log.error("json parse error: %s - %s", e, line)
             except requests.exceptions.HTTPError as e:
                 errors += 1
-                logging.error("caught http error %s on %s try", e, errors)
+                log.error("caught http error %s on %s try", e, errors)
                 if self.http_errors and errors == self.http_errors:
-                    logging.warn("too many errors")
+                    log.warning("too many errors")
                     raise e
                 if e.response.status_code == 420:
                     if interruptible_sleep(errors * 60, event):
-                        logging.info("stopping filter")
+                        log.info("stopping filter")
                         return
                 else:
                     if interruptible_sleep(errors * 5, event):
-                        logging.info("stopping filter")
+                        log.info("stopping filter")
                         return
 
             except Exception as e:
                 errors += 1
-                logging.error("caught exception %s on %s try", e, errors)
+                log.error("caught exception %s on %s try", e, errors)
                 if self.http_errors and errors == self.http_errors:
-                    logging.warn("too many errors")
+                    log.warning("too many errors")
                     raise e
                 if interruptible_sleep(errors, event):
-                    logging.info("stopping filter")
+                    log.info("stopping filter")
+                    return
+
+    def labs_v1_sample(self, event=None, record_keepalive=False):
+        """
+        Returns a small random sample of all public statuses, using the new Twitter
+        labs API version of the endpoint.
+
+        The Tweets returned by the default access level are the same, so if two
+        different clients connect to this endpoint, they will see the same Tweets.
+
+        If a threading.Event is provided for event and the event is set,
+        the sample will be interrupted.
+
+        Requires the use of application only authorisation (set app_auth=True on
+        constructing the Twarc client, or on the commandline).
+        """
+
+        if not self.app_auth:
+            raise RuntimeError(
+                "This endpoint is only available with application authentication. "
+                "Pass app_auth=True in Python or --app-auth on the command line."
+            )
+
+        url = 'https://api.twitter.com/labs/1/tweets/stream/sample'
+        headers = {'accept-encoding': 'deflate, gzip'}
+        errors = 0
+        while True:
+            try:
+                log.info("connecting to labs sample stream")
+                resp = self.get(url, headers=headers, stream=True)
+                errors = 0
+                for raw_line in resp.iter_lines(chunk_size=512):
+                    line = raw_line.decode()
+                    if event and event.is_set():
+                        log.info("stopping sample")
+                        # Explicitly close response
+                        resp.close()
+                        return
+                    if line == "":
+                        log.info("keep-alive")
+                        if record_keepalive:
+                            yield "keep-alive"
+                        continue
+                    try:
+                        yield json.loads(line)
+                    except Exception as e:
+                        log.error("json parse error: %s - %s", e, line)
+            except requests.exceptions.HTTPError as e:
+                errors += 1
+                log.error("caught http error %s on %s try", e, errors)
+                if self.http_errors and errors == self.http_errors:
+                    log.warning("too many errors")
+                    raise e
+                if e.response_status_code == 403:
+                    log.warning("access denied for app (403 Error)")
+                    raise e
+                if e.response.status_code == 420:
+                    if interruptible_sleep(errors * 60, event):
+                        log.info("stopping filter")
+                        return
+                else:
+                    if interruptible_sleep(errors * 5, event):
+                        log.info("stopping filter")
+                        return
+
+            except Exception as e:
+                errors += 1
+                log.error("caught exception %s on %s try", e, errors)
+                if self.http_errors and errors == self.http_errors:
+                    log.warning("too many errors")
+                    raise e
+                if interruptible_sleep(errors, event):
+                    log.info("stopping filter")
                     return
 
     def dehydrate(self, iterator):
@@ -432,9 +609,9 @@ class Twarc(object):
             try:
                 yield json.loads(line)['id_str']
             except Exception as e:
-                logging.error("uhoh: %s\n" % e)
+                log.error("uhoh: %s\n" % e)
 
-    def hydrate(self, iterator):
+    def hydrate(self, iterator, trim_user=False):
         """
         Pass in an iterator of tweet ids and get back an iterator for the
         decoded JSON for each corresponding tweet.
@@ -448,10 +625,12 @@ class Twarc(object):
             tweet_id = tweet_id.strip()  # remove new line if present
             ids.append(tweet_id)
             if len(ids) == 100:
-                logging.info("hydrating %s ids", len(ids))
+                log.info("hydrating %s ids", len(ids))
                 resp = self.post(url, data={
                     "id": ','.join(ids),
-                    "include_ext_alt_text": 'true'
+                    "include_ext_alt_text": 'true',
+                    "include_entities": 'true',
+                    "trim_user": trim_user
                 })
                 tweets = resp.json()
                 tweets.sort(key=lambda t: t['id_str'])
@@ -461,10 +640,12 @@ class Twarc(object):
 
         # hydrate any remaining ones
         if len(ids) > 0:
-            logging.info("hydrating %s", ids)
+            log.info("hydrating %s", ids)
             resp = self.post(url, data={
                 "id": ','.join(ids),
-                "include_ext_alt_text": 'true'
+                "include_ext_alt_text": 'true',
+                "include_entities": 'true',
+                "trim_user": trim_user
             })
             for tweet in resp.json():
                 yield tweet
@@ -475,18 +656,26 @@ class Twarc(object):
         except StopIteration:
             return []
 
-    def retweets(self, tweet_id):
+    def retweets(self, tweet_ids):
         """
-        Retrieves up to the last 100 retweets for the provided
-        tweet.
+        Retrieves up to the last 100 retweets for the provided iterator of tweet_ids.
         """
-        logging.info("retrieving retweets of %s", tweet_id)
-        url = "https://api.twitter.com/1.1/statuses/retweets/""{}.json".format(
-                tweet_id)
+        if not isinstance(tweet_ids, types.GeneratorType):
+            tweet_ids = iter(tweet_ids)
 
-        resp = self.get(url, params={"count": 100})
-        for tweet in resp.json():
-            yield tweet
+        for tweet_id in tweet_ids:
+            if hasattr(tweet_id, 'strip'):
+                tweet_id = tweet_id.strip()
+            log.info("retrieving retweets of %s", tweet_id)
+            url = "https://api.twitter.com/1.1/statuses/retweets/""{}.json".format(
+                    tweet_id)
+            try:
+                resp = self.get(url, params={"count": 100}, allow_404=True)
+                for tweet in resp.json():
+                    yield tweet
+            except requests.exceptions.HTTPError as e:
+                if e.response.status_code == 404:
+                    log.info("can't get tweets for non-existent tweet: %s", tweet_id)
 
     def trends_available(self):
         """
@@ -513,7 +702,7 @@ class Twarc(object):
             resp = self.get(url, params=params, allow_404=True)
         except requests.exceptions.HTTPError as e:
             if e.response.status_code == 404:
-                logging.info("no region matching WOEID %s", woeid)
+                log.info("no region matching WOEID %s", woeid)
             raise e
         return resp.json()
 
@@ -545,17 +734,17 @@ class Twarc(object):
         # get replies to the tweet
         screen_name = tweet['user']['screen_name']
         tweet_id = tweet['id_str']
-        logging.info("looking for replies to: %s", tweet_id)
+        log.info("looking for replies to: %s", tweet_id)
         for reply in self.search("to:%s" % screen_name, since_id=tweet_id):
 
             if reply['in_reply_to_status_id_str'] != tweet_id:
                 continue
 
             if reply['id_str'] in prune:
-                logging.info("ignoring pruned tweet id %s", reply['id_str'])
+                log.info("ignoring pruned tweet id %s", reply['id_str'])
                 continue
 
-            logging.info("found reply: %s", reply["id_str"])
+            log.info("found reply: %s", reply["id_str"])
 
             if recursive:
                 if reply['id_str'] not in prune:
@@ -569,11 +758,11 @@ class Twarc(object):
         # get other potential replies to it
 
         reply_to_id = tweet.get('in_reply_to_status_id_str')
-        logging.info("prune=%s", prune)
+        log.info("prune=%s", prune)
         if recursive and reply_to_id and reply_to_id not in prune:
             t = self.tweet(reply_to_id)
             if t:
-                logging.info("found reply-to: %s", t['id_str'])
+                log.info("found reply-to: %s", t['id_str'])
                 prune = prune + (tweet['id_str'],)
                 for r in self.replies(t, recursive=True, prune=prune):
                     yield r
@@ -581,11 +770,11 @@ class Twarc(object):
         # if this tweet is a quote go get that too whatever tweets it
         # may be in reply to
 
-        quote_id = tweet.get('quotes_status_id_str')
+        quote_id = tweet.get('quoted_status_id_str')
         if recursive and quote_id and quote_id not in prune:
             t = self.tweet(quote_id)
             if t:
-                logging.info("found quote: %s", t['id_str'])
+                log.info("found quote: %s", t['id_str'])
                 prune = prune + (tweet['id_str'],)
                 for r in self.replies(t, recursive=True, prune=prune):
                     yield r
@@ -613,7 +802,7 @@ class Twarc(object):
                 resp = self.get(url, params=params, allow_404=True)
             except requests.exceptions.HTTPError as e:
                 if e.response.status_code == 404:
-                    logging.error("no matching list")
+                    log.error("no matching list")
                 raise e
 
             users = resp.json()
@@ -621,6 +810,23 @@ class Twarc(object):
                 yield user
             params['cursor'] = users['next_cursor']
 
+    def oembed(self, tweet_url, **params):
+        """
+        Returns the oEmbed JSON for a tweet. The JSON includes an html
+        key that contains the HTML for the embed. You can pass in
+        parameters that correspond to the paramters that Twitter's
+        statuses/oembed endpoint supports. For example:
+
+        o = client.oembed('https://twitter.com/biz/status/21', theme='dark')
+        """
+        log.info("generating embedding for tweet %s", tweet_url)
+        url = "https://publish.twitter.com/oembed"
+
+        params['url'] = tweet_url
+        resp = self.get(url, params=params)
+
+        return resp.json()
+
     @rate_limit
     @catch_conn_reset
     @catch_timeout
@@ -629,32 +835,39 @@ class Twarc(object):
         if not self.client:
             self.connect()
 
-        if "params" in kwargs:
-            kwargs["params"]["tweet_mode"] = self.tweet_mode
+        # set default tweet_mode
+        if "params" not in kwargs:
+            kwargs["params"] = {"tweet_mode": self.tweet_mode}
         else:
-            kwargs["params"] = {"tweet_node": self.tweet_mode}
+            kwargs["params"]["tweet_mode"] = self.tweet_mode
+
+        # override tweet_mode for labs and premium endpoints
+        if re.search(r"api.twitter.com/labs", args[0]):
+            kwargs["params"] = {"format": "detailed"}
+        elif re.search(r"api.twitter.com/1.1/tweets/search/", args[0]):
+            kwargs["params"].pop("tweet_mode")
 
         # Pass allow 404 to not retry on 404
         allow_404 = kwargs.pop('allow_404', False)
         connection_error_count = kwargs.pop('connection_error_count', 0)
         try:
-            logging.info("getting %s %s", args, kwargs)
+            log.info("getting %s %s", args, kwargs)
             r = self.last_response = self.client.get(*args, timeout=(3.05, 31),
                                                      **kwargs)
             # this has been noticed, believe it or not
             # https://github.com/edsu/twarc/issues/75
             if r.status_code == 404 and not allow_404:
-                logging.warn("404 from Twitter API! trying again")
+                log.warning("404 from Twitter API! trying again")
                 time.sleep(1)
                 r = self.get(*args, **kwargs)
             return r
-        except requests.exceptions.ConnectionError as e:
+        except (ssl.SSLError, ConnectionError, ProtocolError) as e:
             connection_error_count += 1
-            logging.error("caught connection error %s on %s try", e,
+            log.error("caught connection error %s on %s try", e,
                           connection_error_count)
             if (self.connection_errors and
                     connection_error_count == self.connection_errors):
-                logging.error("received too many connection errors")
+                log.error("received too many connection errors")
                 raise e
             else:
                 self.connect()
@@ -675,23 +888,24 @@ class Twarc(object):
 
         connection_error_count = kwargs.pop('connection_error_count', 0)
         try:
-            logging.info("posting %s %s", args, kwargs)
+            log.info("posting %s %s", args, kwargs)
             self.last_response = self.client.post(*args, timeout=(3.05, 31),
                                                   **kwargs)
             return self.last_response
-        except requests.exceptions.ConnectionError as e:
+        except (ssl.SSLError, ConnectionError, ProtocolError) as e:
             connection_error_count += 1
-            logging.error("caught connection error %s on %s try", e,
+            log.error("caught connection error %s on %s try", e,
                           connection_error_count)
             if (self.connection_errors and
                     connection_error_count == self.connection_errors):
-                logging.error("received too many connection errors")
+                log.error("received too many connection errors")
                 raise e
             else:
                 self.connect()
                 kwargs['connection_error_count'] = connection_error_count
                 return self.post(*args, **kwargs)
 
+    @catch_timeout
     def connect(self):
         """
         Sets up the HTTP session to talk to Twitter. If one is active it is
@@ -702,19 +916,31 @@ class Twarc(object):
             raise RuntimeError("MissingKeys")
 
         if self.client:
-            logging.info("closing existing http session")
+            log.info("closing existing http session")
             self.client.close()
         if self.last_response:
-            logging.info("closing last response")
+            log.info("closing last response")
             self.last_response.close()
-        logging.info("creating http session")
+        log.info("creating http session")
 
-        self.client = OAuth1Session(
-            client_key=self.consumer_key,
-            client_secret=self.consumer_secret,
-            resource_owner_key=self.access_token,
-            resource_owner_secret=self.access_token_secret
-        )
+        if not self.app_auth:
+            logging.info('creating OAuth1 user authentication')
+            self.client = OAuth1Session(
+                client_key=self.consumer_key,
+                client_secret=self.consumer_secret,
+                resource_owner_key=self.access_token,
+                resource_owner_secret=self.access_token_secret
+            )
+        else:
+            logging.info('creating OAuth2 app authentication')
+            client = BackendApplicationClient(client_id=self.consumer_key)
+            oauth = OAuth2Session(client=client)
+            token = oauth.fetch_token(
+                token_url='https://api.twitter.com/oauth2/token',
+                client_id=self.consumer_key,
+                client_secret=self.consumer_secret
+            )
+            self.client = oauth
 
     def get_keys(self):
         """
@@ -747,24 +973,31 @@ class Twarc(object):
         keys_present = self.consumer_key and self.consumer_secret and \
                        self.access_token and self.access_token_secret
 
+        if self.app_auth:
+            # no need to validate keys when using OAuth2 App Auth.
+            return True
+
         if keys_present:
             try:
                 # Need to explicitly reconnect to confirm the current creds
                 # are used in the session object.
                 self.connect()
                 self.get(url)
+                return True
             except requests.HTTPError as e:
                 if e.response.status_code == 401:
                     raise RuntimeError('Invalid credentials provided.')
                 else:
                     raise e
         else:
-            raise RuntimeError('Incomplete credentials provided.')
+            print('Incomplete credentials provided.')
+            print('Please run the command "twarc configure" to get started.')
+            sys.exit()
 
     def load_config(self):
         path = self.config
         profile = self.profile
-        logging.info("loading %s profile from config %s", profile, path)
+        log.info("loading %s profile from config %s", profile, path)
 
         if not path or not os.path.isfile(path):
             return {}
