@@ -1,6 +1,13 @@
 import time
 import logging
-import requests
+
+from requests import HTTPError
+from requests.packages.urllib3.exceptions import ReadTimeoutError
+from requests.exceptions import ChunkedEncodingError, ReadTimeout, \
+                                ContentDecodingError
+
+log = logging.getLogger('twarc')
+
 
 def rate_limit(f):
     """
@@ -22,11 +29,11 @@ def rate_limit(f):
                 # with a new message from the old error.
                 try:
                     resp.raise_for_status()
-                except requests.HTTPError as e:
+                except HTTPError as e:
                     message = "\nThis is a protected or locked account, or" +\
                               " the credentials provided are no longer valid."
                     e.args = (e.args[0] + message,) + e.args[1:]
-                    logging.warn("401 Authentication required for %s", resp.url)
+                    log.warning("401 Authentication required for %s", resp.url)
                     raise
             elif resp.status_code == 429:
                 reset = int(resp.headers['x-rate-limit-reset'])
@@ -34,15 +41,15 @@ def rate_limit(f):
                 seconds = reset - now + 10
                 if seconds < 1:
                     seconds = 10
-                logging.warn("rate limit exceeded: sleeping %s secs", seconds)
+                log.warning("rate limit exceeded: sleeping %s secs", seconds)
                 time.sleep(seconds)
             elif resp.status_code >= 500:
                 errors += 1
                 if errors > 30:
-                    logging.warn("too many errors from Twitter, giving up")
+                    log.warning("too many errors from Twitter, giving up")
                     resp.raise_for_status()
                 seconds = 60 * errors
-                logging.warn("%s from Twitter API, sleeping %s",
+                log.warning("%s from Twitter API, sleeping %s",
                              resp.status_code, seconds)
                 time.sleep(seconds)
             else:
@@ -54,6 +61,7 @@ def catch_conn_reset(f):
     """
     A decorator to handle connection reset errors even ones from pyOpenSSL
     until https://github.com/edsu/twarc/issues/72 is resolved
+    It also handles ChunkedEncodingError which has been observed in the wild.
     """
     try:
         import OpenSSL
@@ -66,8 +74,8 @@ def catch_conn_reset(f):
         if ConnectionError:
             try:
                 return f(self, *args, **kwargs)
-            except ConnectionError as e:
-                logging.warn("caught connection reset error: %s", e)
+            except (ConnectionError, ChunkedEncodingError) as e:
+                log.warning("caught connection reset error: %s", e)
                 self.connect()
                 return f(self, *args, **kwargs)
         else:
@@ -82,8 +90,8 @@ def catch_timeout(f):
     def new_f(self, *args, **kwargs):
         try:
             return f(self, *args, **kwargs)
-        except requests.exceptions.ReadTimeout as e:
-            logging.warn("caught read timeout: %s", e)
+        except (ReadTimeout, ReadTimeoutError) as e:
+            log.warning("caught read timeout: %s", e)
             self.connect()
             return f(self, *args, **kwargs)
     return new_f
@@ -97,8 +105,8 @@ def catch_gzip_errors(f):
     def new_f(self, *args, **kwargs):
         try:
             return f(self, *args, **kwargs)
-        except requests.exceptions.ContentDecodingError as e:
-            logging.warn("caught gzip error: %s", e)
+        except ContentDecodingError as e:
+            log.warning("caught gzip error: %s", e)
             self.connect()
             return f(self, *args, **kwargs)
     return new_f
@@ -110,7 +118,7 @@ def interruptible_sleep(t, event=None):
 
     Returns True if interrupted
     """
-    logging.info("sleeping %s", t)
+    log.info("sleeping %s", t)
 
     if event is None:
         time.sleep(t)
